function varargout=run(inputfile,varargin)
% FUNCTION fc_oogmsh.gmsh.run
%   Runs gmsh command
%
% USAGE:
%   meshfile=fc_oogmsh.gmsh.run(inputfile)
%   meshfile=fc_oogmsh.gmsh.run(inputfile,key,value)
%
% INPUT:
%   inputfile: name of a .geo file or a mesh file as string.
%
%   optional parameters as key/value pairs:
%     'verbose' : to specify verbosity.
%     'meshfile': to specify the name of the output mesh file.
%     'force'   : to force overwriting of the output mesh.
%     'd'       : to specify space dimension 2(default) or 3.
%     'strings' : cells as input arguments to gmsh command '-string "..."', 
%                 for each cell with "..." is the string cell value.
%
% OUTPUT:
%   meshfile: name of the output mesh file.
%
% SAMPLES:
%
% Copyright (C) 2017-2020 F. Cuvelier
  env=fc_oogmsh.environment();
  format=fc_oogmsh.gmsh.setformat();
  p = inputParser; 
 % p.KeepUnmatched=true;
  p.addParamValue('d',2,@isscalar); % d-simplex
  p.addParamValue('options','',@ischar); %gmsh option
  p.addParamValue('Loptions',{},@iscell); %gmsh list of options
  p.addParamValue('meshfile','',@ischar); % output msh file
  p.addParamValue('strings',{},@iscell);% -string "<strings>"
  %p.addParamValue('check',false,@islogical); % use gmsh to check mesh file
  p.addParamValue('verbose',2,@isscalar);
  %p.addParamValue('MshFileVersion',format,@(x) ismember(x,{'2.2','4.0','4.1'}));
  p.addParamValue('MshFileVersion',format,@(x) ismember(x,fc_oogmsh.gmsh.getMSHfile_formats()));
  p.addParamValue('force',false,@islogical); % force writing file
  p.addParamValue('fprintf',[]); % not used
  p.addParamValue('pathrep',@(s) strrep(s,env.Path,'<fc-oogmsh>'))
  p.parse(varargin{:});
  R=p.Results;
  if isempty(R.pathrep), R.pathrep=@(s) s;end
  gmsh_version=fc_oogmsh.gmsh.version();
  ver=strsplit(gmsh_version,'.');
  %if str2num(ver{1}) >= 4, R.strings{end+1}=sprintf('Mesh.MshFileVersion="%s";',R.MshFileVersion);end % Force mesh file format to version 2 for gmsh version >=4
  %if length(R.MshFileVersion)>0 % Otherwise use default format
  fc_oogmsh.gmsh.checkformat(R.MshFileVersion);
  R.strings{end+1}=sprintf('Mesh.MshFileVersion=%s;',R.MshFileVersion);
  %end
  % Check for inputfile
  [fid,message]=fopen(inputfile,'r');
  if ( fid == -1 ), error([message,' : ',inputfile]); end
  fclose(fid);
  
  % 
  if isempty(R.meshfile)
    [pathstr,name,ext] = fileparts(inputfile);
    R.meshfile=[pathstr,filesep,name,'.msh'];
    if strcmp(R.meshfile,inputfile), R.meshfile=[pathstr,filesep,name,'_new.msh'];end
  end
  %R.meshfile=strrep(R.meshfile,pwd,'.');
  %inputfile=strrep(inputfile,pwd,'.');
  %if R.verbose>=1, fprintf('[fc-oogmsh] Input file : %s\n',strrep(inputfile,pwd,'.'));end
  if R.verbose>=1, fprintf('[fc-oogmsh] Input file : %s\n',R.pathrep(inputfile));end
  
  if ((fc_tools.sys.isfileexists(R.meshfile)) && (R.force==false))
    status=0;
    if R.verbose>=1
      MeshFormat=fc_oogmsh.msh.read_MeshFormat(R.meshfile);
      fprintf('[fc-oogmsh] Mesh file %s [version %s] already exists.\n  -> Use "force" flag to rebuild if needed.\n',R.pathrep(R.meshfile),MeshFormat.version);
    end
    if nargout==1,varargout{1}=R.meshfile;end
    return
  end
  
  gmsh_exe=fc_oogmsh.gmsh.command();
  
  if ((fc_tools.sys.isfileexists(R.meshfile)) && (R.force==true))
    status=0;
    if R.verbose>=1, fprintf('[fc-oogmsh] Overwritting mesh file %s\n',R.pathrep(R.meshfile));end
  end
  stringOption='';
  if ~isempty(R.strings)
    stringOption=sprintf('-string "%s" ',strcat(R.strings{:})); 
  end
  Options='';
  if ~isempty(R.Loptions)
    Options=sprintf(' %s ',R.Loptions{:}); 
  end
  
  %gmsh_cmd=sprintf('%s -%d %s %s %s %s -o %s',gmsh_exe,R.d,R.options,stringOption,Options,inputfile,R.meshfile);
  gmsh_cmd=sprintf('%s -%d %s %s %s %s -o %s',gmsh_exe,R.d,R.options,stringOption,Options,inputfile,R.meshfile);
 
  version=fc_oogmsh.gmsh.version();
  gmsh_print=sprintf('gmsh -%d %s %s %s %s -o %s',R.d,R.options,stringOption,Options, ...
                     R.pathrep(inputfile),R.pathrep(R.meshfile));
  if R.verbose>=2
    fprintf('[fc-oogmsh] Starting building mesh %s with gmsh %s\n[fc-oogmsh]  Using command : %s\n Be patient...\n',R.pathrep(R.meshfile),version,gmsh_print);
  end
  
  if R.verbose<=2, gmsh_cmd=[gmsh_cmd,' -v 0 '];end % Added for windows
  if R.verbose>2
    fprintf('[fc-oogmsh] gmsh output :\n')
  end
  [status,result]=system(gmsh_cmd);
  if R.verbose>2
    disp(R.pathrep(result))
  end
  if status~=0, 
    fprintf('%s\n',result);
    fprintf('command : %s\n',gmsh_print)
    error('GMSH command failed');
  end
  
  if nargout==1,varargout{1}=R.meshfile;end
  fprintf('[fc-oogmsh] Using gmsh %s to write MSH file format version %s in %s\n',gmsh_version,R.MshFileVersion,R.pathrep(R.meshfile))
end
