function varargout=sliceiso(Th,u,P,varargin)
% FUNCTION sliceiso 
%   Plots isolines from scalar data values on the intersection of the plane 
%   defined by
%         P(1)*x+P(2)*y+P(3)*z+P(4)=0
%   and the siMesh object.
%
% USAGE
%   fc_siplt.sliceiso(Th,u,P) or sliceiso(Th,u,P) 
%     where 
%     - Th is a siMesh object 
%     - u is a vector of length Th.nq which contains the scalar data values 
%       on each vertices of the siMesh object.
%     - P contains the 4   coefficients of the plane equation.
%     This function only acts on the 3D meshes.
%
%   fc_siplt.sliceiso(Th,u,key,value,...) allows additional key/value pairs to be 
%     used when displaying u. The key strings could be
%
%     'labels' to specify the labels of the 2-simplex elementary meshes. 
%              Default [] i.e all.
%     'niso'   number of isolines to draw. The associated isovalues are
%              taken as linspace(min(u),max(u),niso).Default value is 10.
%     'isorange' to manualy specify all the isovalues.
%     'color'  to specify a same color for all the isolines.
%     'isocolorbar' if true display a colorbar.Default is false.
%     'format' to specify the format of the isovalues print in the colorbar.
%              Default is '%g'
%
%   or any options of the plot3 function.
%
%   varargout=fc_siplt.sliceiso(Th,...)
%     This function accepts until 4 output arguments :
%     - 1st output is the colors of the isolines
%     - 2nd output is the isovalues of the isolines
%     - 3th output is the handle of the colobar iso.
%     - 4th output is all the handles of the isolines as an 2D-array of 
%       dimension N-by-niso, where N is the number of 2-simplex elementary
%       meshes where isolines are drawn.
% 
% SAMPLES
%   Th=fc_simesh.HyperCube(3,10);
%   U=Th.eval(@(x,y,z) cos(2*x-y+z).*sin(x+y-3*z));
%   figure(1)
%   fc_siplt.plotiso(Th,U)
%   figure(2)
%   fc_siplt.plotiso(Th,U,'labels',[1,2,5],'color','k')
%
%    Copyright (C) 2017-2010 F. Cuvelier
  assert(fc_simesh.is_siMesh(Th))
  assert(Th.d==3 && Th.dim==3);
  U=Th.eval(u); % also verifies dimensions...
  p = inputParser; 
  p.KeepUnmatched=true; 
  p.PartialMatching=false;
  p.addParamValue('labels',[]); % select labels to draw
  p.addParamValue('niso',10,@isscalar);
  p.addParamValue('isorange',[]);
  p.addParamValue('isocolorbar',false,@islogical);
  p.addParamValue('format','%g',@ischar);
  p.addParamValue('color',[]); % for unique color
  p.addParamValue('mouse',true,@islogical);
  p.addParamValue('merge',true,@islogical);
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  
  if isempty(R.labels), R.labels=Th.sThlab(Th.sThsimp==3);end
  if isempty(R.labels), warning('No label found for %d-simplex sub-meshes',3);return;end
%    if isempty(R.isorange)
%      R.isorange=linspace(min(u),max(u),R.niso);
%    end
%    R.niso=length(R.isorange); 
%    if isempty(R.color)
%      cmap=colormap();%R.colormap);
%      t=0:1/(R.niso-1):1;
%      colors = fc_tools.graphics.RGB(t,cmap);
%    else
%      if ischar(R.color), R.color = fc_tools.graphics.colorstr2rgb(R.color);end
%      colors=ones(R.niso,1)*R.color;
%    end
  umin=min(U);umax=max(U);
  if isempty(R.isorange)
    R.isorange=linspace(umin,umax,R.niso);
  else
    R.isorange=sort(R.isorange);
  end
  [R.isorange,colors]=fc_graphics4mesh.getisocolors(U,R.isorange,R.color);
  R.niso=length(R.isorange);
  
  LB=Th.find(3);
  hfig=gcf();
  PrevHold=fc_tools.graphics.SetHoldOn(true);
  
  if R.merge
    [q,me,toGlobal]=Th.get_mesh('labels',R.labels);
    [~,~,~,h]=fc_graphics4mesh.sliceiso(q,me,U(toGlobal),P,'isorange',R.isorange, ...
                              'color',colors, varargin{:});
  else
    h=zeros(length(R.labels),R.niso);
    for l=1:length(R.labels)
      [I,K]=ismember(R.labels(l),Th.sThlab(LB));
      if K==0
        warning('label %d not found for %d-simplex sub-meshes',R.labels(l),2);
      else
        ilab=LB(K);
        sTh=Th.sTh{ilab};
        [~,~,~,h(l,:)]=fc_graphics4mesh.sliceiso(sTh.q,sTh.me,sTh.getLocalData(U),P,'niso',R.niso,'isorange',R.isorange, ...
                                              'color',colors,varargin{:});
                                    
      end
    end
  end
  cax=[];
  if R.isocolorbar
    cax=fc_tools.graphics.colorbarIso(colors,R.isorange,R.format);
  end
  if R.mouse && ~fc_tools.comp.isOctave()
    datacursormode on
    dcm = datacursormode(gcf);
    set(dcm,'UpdateFcn',@fc_simesh.showmeshinfo)
  end
  if nargout>=1, varargout{1}=colors;end
  if nargout>=2, varargout{2}=R.isorange;end
  if nargout>=3, varargout{3}=cax;end
  if nargout==4, varargout{4}=h;end
  fc_tools.graphics.RestoreHold(PrevHold)
  figure(hfig);
end
